import _assign from 'lodash/assign';
import _find from 'lodash/find';
import _debounce from 'lodash/debounce';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from 'react';

import { PayloadStates } from 'lore-utils';
import { Field } from 'lore-react-forms';
import onClickOutside from 'react-onclickoutside';

var AutoCompleteField = function (_Field) {
  _inherits(AutoCompleteField, _Field);

  function AutoCompleteField(props) {
    _classCallCheck(this, AutoCompleteField);

    // bind component methods
    var _this = _possibleConstructorReturn(this, (AutoCompleteField.__proto__ || Object.getPrototypeOf(AutoCompleteField)).call(this, props));

    _this.handleUpdateInput = _this.handleUpdateInput.bind(_this);
    _this.handleNewRequest = _this.handleNewRequest.bind(_this);
    _this.updateOptions = _this.updateOptions.bind(_this);
    _this.renderOption = _this.renderOption.bind(_this);
    _this.filter = _this.filter.bind(_this);

    // setup debounce if configured
    var debounceDelay = _this.props.debounceDelay;
    if (debounceDelay > 0) {
      _this.updateOptions = _debounce(_this.updateOptions, debounceDelay);
    }

    // if the value exists in the data, use it to set the initial search text
    var initialOption = _find(props.options.data, function (option) {
      return option.id === props.data[props.name];
    });
    var searchText = initialOption ? initialOption.data[props.field] : '';

    _this.state = {
      searchText: searchText,
      isModified: false,
      options: _this.props.options,
      hasFocus: false
    };
    return _this;
  }

  _createClass(AutoCompleteField, [{
    key: 'onFocus',
    value: function onFocus() {
      this.setState({
        hasFocus: true
      });
    }
  }, {
    key: 'onBlur',
    value: function onBlur() {}
  }, {
    key: 'handleClickOutside',
    value: function handleClickOutside(event) {
      if (this.state.hasFocus) {
        this.setState({
          hasFocus: false,
          touched: true
        });
      }
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        options: this.filter(this.state.searchText)
      });
    }
  }, {
    key: 'filter',
    value: function filter(searchText) {
      if (this.props.filter) {
        return this.props.filter(searchText);
      }

      return _assign({}, this.state.options, {
        state: PayloadStates.RESOLVED
      });
    }

    /*
     * Change options based on user input
     */

  }, {
    key: 'updateOptions',
    value: function updateOptions(searchText) {
      this.setState({
        options: this.filter(searchText)
      });
    }
  }, {
    key: 'handleUpdateInput',
    value: function handleUpdateInput(e) {
      var searchText = e.target.value;

      this.setState({
        searchText: searchText,
        isModified: true
      });

      if (this.props.debounceDelay > 0) {
        this.setState({
          options: _assign({}, this.state.options, {
            state: PayloadStates.FETCHING
          })
        });
      }

      this.updateOptions(searchText);
    }
  }, {
    key: 'handleNewRequest',
    value: function handleNewRequest(item, index) {
      this.props.onChange(this.props.name, item.value);
      this.setState({
        searchText: item.text,
        hasFocus: false
      });
      this.handleUpdateInput({
        target: {
          value: item.text
        }
      });
    }
  }, {
    key: 'renderOption',
    value: function renderOption(option) {
      var _this2 = this;

      return React.createElement(
        'li',
        {
          key: option.value,
          style: { cursor: 'pointer' },
          onClick: function onClick() {
            _this2.handleNewRequest(option);
          }
        },
        React.createElement(
          'a',
          null,
          option.text
        )
      );
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props,
          name = _props.name,
          errors = _props.errors,
          hintText = _props.hintText,
          label = _props.label,
          option = _props.option,
          field = _props.field;
      var _state = this.state,
          touched = _state.touched,
          isModified = _state.isModified,
          options = _state.options;


      var error = errors[name];
      var displayError = touched && error;

      var searchText = option ? isModified ? this.state.searchText : option.data[field] : this.state.searchText;

      var suggestions = null;

      function mapDataToOptions(data) {
        return data.map(function (datum) {
          return {
            value: datum.id,
            text: datum.data[field]
          };
        });
      }

      if (options.state === PayloadStates.FETCHING) {
        suggestions = React.createElement(
          'li',
          { className: 'disabled' },
          React.createElement(
            'a',
            null,
            'Searching...'
          )
        );
      } else if (options.data.length > 0) {
        suggestions = mapDataToOptions(options.data).map(this.renderOption);
      } else {
        suggestions = React.createElement(
          'li',
          { className: 'disabled' },
          React.createElement(
            'a',
            null,
            'No results'
          )
        );
      }

      var className = 'form-group dropdown';
      if (this.state.hasFocus) {
        className += ' open';
      }
      if (displayError) {
        className += ' has-error';
      }

      return React.createElement(
        'div',
        { className: className },
        label ? React.createElement(
          'label',
          null,
          label
        ) : null,
        React.createElement('input', {
          type: 'text',
          value: searchText,
          onChange: this.handleUpdateInput,
          onFocus: this.onFocus,
          onBlur: this.onBlur,
          className: 'form-control',
          placeholder: hintText
        }),
        React.createElement(
          'ul',
          { className: 'dropdown-menu col-xs-12' },
          suggestions
        ),
        displayError ? React.createElement(
          'span',
          { className: 'help-block' },
          error
        ) : null
      );
    }
  }]);

  return AutoCompleteField;
}(Field);

AutoCompleteField.propTypes = _assign({}, {
  options: React.PropTypes.object.isRequired,
  debounceDelay: React.PropTypes.number,
  filter: React.PropTypes.func
});

AutoCompleteField.defaultProps = _assign({}, {
  options: {
    data: []
  },
  debounceDelay: 250
});

export default onClickOutside(AutoCompleteField);